""" The enum defining the log levels """

from enum import Enum


class LogLevel(Enum):
    """ The enum defining the log levels """

    highlight = 4
    debug = 3
    info = 2
    warning = 1
    error = 0

    @property
    def ansi_color(self):
        """ Associates an ansi color with each log level """
        if self == LogLevel.highlight:
            return '35m'
        elif self == LogLevel.debug:
            return '34m'
        elif self == LogLevel.info:
            return '32m'
        elif self == LogLevel.warning:
            return '33m'
        elif self == LogLevel.error:
            return '31m'

    @property
    def ansi_color_sequence(self):
        return u'\u001b' + '[' + str(self.ansi_color)

    def __lt__(self, other):
        if self.__class__ is other.__class__:
            return self.value < other.value
        return NotImplemented

    def __le__(self, other):
        if self.__class__ is other.__class__:
            return self.value <= other.value
        return NotImplemented



"""  Logger abstract class """

from abc import ABC, abstractmethod

from QLog.log_entry import LogEntry
from QLog.log_level import LogLevel


class Logger(ABC):
    """  Logger abstract class """

    log_level: LogLevel

    @abstractmethod
    def do_log(self, log_entry):
        """ Log function to implement """

        pass

    def log(self, log_entry: LogEntry):
        """ Log function to use """

        if log_entry.log_level <= self.log_level:
            self.do_log(log_entry)



import QLog
from QLog.console_logger import ConsoleLogger
from QLog import QLogHighlight, QLogDebug, QLogInfo, QLogWarning, QLogError


def main():
    QLog.loggers = [ConsoleLogger()]
    QLogHighlight('Test')
    QLogDebug('Test')
    QLogInfo('Test')
    QLogWarning('Test')
    QLogError('Test')


if __name__ == '__main__':
    main()



"""  Logger to log into console """

from QLog.log_entry import LogEntry
from QLog.log_level import LogLevel
from QLog.logger import Logger


class ConsoleLogger(Logger):
    """  Logger to log into console """

    def __init__(self, log_level=LogLevel.highlight):
        self.log_level = log_level

    escape = u'\u001b'
    ansi_bold = '1m'
    ansi_reset = '0m'
    ansi_text_color = '37m'

    ansi_bold_sequence = escape + '[' + ansi_bold
    ansi_reset_sequence = escape + '[' + ansi_reset
    ansi_text_sequence = escape + '[' + ansi_text_color

    def do_log(self, log_entry: LogEntry):
        print(
            ConsoleLogger.ansi_bold_sequence +
            ConsoleLogger.ansi_text_sequence +
            log_entry.meta_text +
            ConsoleLogger.ansi_bold_sequence +
            log_entry.log_level.ansi_color_sequence +
            log_entry.text +
            ConsoleLogger.ansi_reset_sequence
        )



""" QLog logger """

import inspect
import os
from datetime import datetime

from QLog.log_entry import LogEntry

__version__ = "0.1.0"

from QLog.log_level import LogLevel
from QLog.logger import Logger


def QLogHighlight(data):
    log(LogLevel.highlight, data)


def QLogDebug(data):
    log(LogLevel.debug, data)


def QLogInfo(data):
    log(LogLevel.info, data)


def QLogWarning(data):
    log(LogLevel.warning, data)


def QLogError(data):
    log(LogLevel.error, data)


loggers: [Logger] = []


def log(level: LogLevel, data):
    caller = inspect.stack()[2]
    entry = LogEntry(
        datetime.now(),
        os.path.splitext(os.path.basename(caller.filename))[0],
        caller.function,
        caller.lineno,
        level,
        data
    )
    for active_logger in loggers:
        active_logger.log(entry)



"""  The log entry to be used for logging """

from datetime import date
from typing import NamedTuple

from QLog.log_level import LogLevel


class LogEntry(NamedTuple):
    """  The log entry to be used for logging """

    date: date
    file: str
    function: str
    line: int
    log_level: LogLevel
    text: str

    @property
    def meta_text(self):
        """  The meta data of a log entry containing date, file, line and function """

        return str(self.date) + ': ' + self.file + ':' + str(self.line) + ' ' + self.function + ': '



"""Setup script for QLog """

import os.path
from setuptools import setup

HERE = os.path.abspath(os.path.dirname(__file__))

with open(os.path.join(HERE, "README.md")) as fid:
    README = fid.read()

setup(
    name="QLog",
    version="0.1.0",
    description="Logger",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/QuantumApplications/QLog-Python",
    author="Quantum Applications",
    author_email="oberdoerfer@quantum.qa",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8"
    ],
    packages=["QLog"],
    include_package_data=True,
    install_requires=[]
)
